import 'dart:convert';

import 'package:code_climate/code_climate.dart';
import 'package:code_climate/src/utils.dart';

class FlutterParser {
  List<Issue> parseFlutterAnalyzeOutput(String input) {
    final lineSplitter = LineSplitter();
    final lines = lineSplitter.convert(input);
    if (lines.isEmpty) {
      return [];
    }
    return lines.where(_filter).map(_parseFlutterAnalyzeLine).toList();
  }

  bool _filter(String line) {
    if (line.trim().isEmpty) {
      return false;
    }
    if (line.startsWith('Analyzing')) {
      return false;
    }
    if (line.startsWith('No issues found!')) {
      return false;
    }
    if (line.contains('issue found.')) {
      return false;
    }

    if (line.contains('issues found.')) {
      return false;
    }

    return true;
  }

  /// example line `   info • The declaration '_unusedMethod' isn't referenced • lib/app.dart:12:8 • unused_element`
  Issue _parseFlutterAnalyzeLine(String analyzerLine) {
    final segments = analyzerLine.split(' • ');

    final severity = segments[0].trim();
    final description = segments[1];
    final name = segments.last.trim();

    final locationSegments = segments[2].split(':');
    final path = locationSegments[0];
    final line = int.parse(locationSegments[1]);
    final beginColumn = int.parse(locationSegments[2]);

    final content =
        'See [$name](https://dart-lang.github.io/linter/lints/$name.html) or if it\'s not avaiable see [lint overview](https://dart-lang.github.io/linter/lints/index.html)';

    return Issue(
      severity: parseFlutterSeverity(severity),
      checkName: name,
      content: content,
      description: description,
      fingerprint: md5(analyzerLine),
      categories: ['Bug Risk', 'Clarity'],
      location: Location(
        path: path,
        beginLine: line,
        beginColumn: beginColumn,
        endLine: line,
        endColumn: beginColumn + 1,
      ),
    );
  }

  List<Issue> parseFlutterFormatOutput(String input) {
    final lineSplitter = LineSplitter();
    final lines = lineSplitter.convert(input);
    if (lines.isEmpty) {
      return [];
    }
    return lines.map(_parseFlutterFormatLine).toList();
  }

  /// example line `lib/app.dart`
  Issue _parseFlutterFormatLine(String analyzerLine) {
    return Issue(
      severity: 'error',
      checkName: 'style',
      categories: ['Style'],
      location: Location(
        path: analyzerLine,
        beginLine: 0,
        beginColumn: 0,
        endLine: 0,
        endColumn: 0,
      ),
      description: 'Please format file $analyzerLine',
      content: 'Run `flutter format .` to fix this issue.',
      fingerprint: md5(analyzerLine),
    );
  }
}
