import 'package:code_climate/src/issue.dart';
import 'package:code_climate/src/parser.dart';
import 'package:process/process.dart';

class FlutterRunner {
  FlutterRunner({this.processManager = const LocalProcessManager()});

  final ProcessManager processManager;

  List<Issue> run() {
    if (!processManager.canRun('flutter')) {
      throw Exception('Can\'t run flutter commands');
    }

    final issues = <Issue>[];
    final flutterParser = FlutterParser();

    final formatResult =
        processManager.runSync(['flutter', 'format', '--no-color', '-n', '.']);
    final formatIssues =
        flutterParser.parseFlutterFormatOutput(formatResult.stdout);
    issues.addAll(formatIssues);

    final analyzeResult =
        processManager.runSync(['flutter', 'analyze', '--no-color']);
    final analyzeIssues =
        flutterParser.parseFlutterAnalyzeOutput(analyzeResult.stdout);
    issues.addAll(analyzeIssues);

    return issues;
  }
}
