import 'dart:convert';
import 'dart:io';
import 'package:crypto/crypto.dart' as crypto show md5;

/// Map dartanalyzer severities to proper Code Climate severities. It is highly
/// subjective but following solution presents the most balanced approach.
String parseSeverity(String analyzerSeverity) {
  switch (analyzerSeverity) {
    case 'INFO':
      return 'info';
    case 'WARNING':
      return 'minor';
    case 'ERROR':
      return 'blocker';
  }

  return 'blocker';
}

/// Map `flutter analyze` severities to proper Code Climate severities. It is highly
/// subjective but following solution presents the most balanced approach.
String parseFlutterSeverity(String analyzerSeverity) {
  switch (analyzerSeverity.toLowerCase()) {
    case 'info':
      return 'info';
    case 'warning':
      return 'minor';
    case 'error':
      return 'blocker';
  }

  return 'blocker';
}

/// Simple md5 hash of inputted [String]
String md5(String input) => crypto.md5.convert(utf8.encode(input)).toString();

/// Dart Analysis Server returns paths in absolute form so
/// we must to relativize it
String relativizePath(String input) {
  if (input.startsWith(Directory.current.path)) {
    return input.replaceFirst('${Directory.current.path}/', '');
  }

  return input;
}
